// Fill out your copyright notice in the Description page of Project Settings.


#include "Player/LMPlayer.h"

#include "KismetTraceUtils.h"
#include "LMUtils.h"
#include "Camera/CameraComponent.h"
#include "Ammo/LMAmmo.h"
#include "Player/LMBulletInfo.h"
#include "Player/LMHitBox.h"
#include "Weapon/LMWeaponManager.h"

ALMPlayer::ALMPlayer()
{
 	// Set this character to call Tick() every frame.  You can turn this off to improve performance if you don't need it.
	PrimaryActorTick.bCanEverTick = true;

	Camera = CreateDefaultSubobject<UCameraComponent>(TEXT("Camera"));
	Camera->SetupAttachment(RootComponent);
	Camera->bUsePawnControlRotation = true;
	Camera->SetRelativeLocation(FVector(20, 0, 90));
	
	ArmsMesh = CreateDefaultSubobject<USkeletalMeshComponent>(TEXT("Arms Mesh"));
	ArmsMesh->SetupAttachment(Camera);

	SpreadStream = FRandomStream(FMath::Rand());
}

// Called when the game starts or when spawned
void ALMPlayer::BeginPlay()
{
	Super::BeginPlay();
	
}

void ALMPlayer::PickUpAmmo(ALMAmmo* Ammo)
{
	if (GEngine)
	{
		FString AmmoAmount = FString::Printf(TEXT("Quantité de munition : %i"), Ammo->GetAmmoAmount());
		GEngine->AddOnScreenDebugMessage(1, 30.f, FColor::Red, AmmoAmount);
	}
	WeaponManager->AddAmmoType(Ammo->GetAmmoType(), Ammo->GetAmmoAmount());
}

int ALMPlayer::GetAmmoCount(const EAmmoType AmmoType) const
{
	return WeaponManager->GetAmmoCount(AmmoType);
}

int ALMPlayer::RemoveAmmo(const EAmmoType AmmoType, const int Count) const
{
	return WeaponManager->RemoveAmmo(AmmoType, Count);
}

void ALMPlayer::SetWeaponManager(ULMWeaponManager* Manager)
{
	UE_LOG(LogTemp, Warning, TEXT("Set weapon manager"))
	if (!Manager)
	{
		GEngine->AddOnScreenDebugMessage(INDEX_NONE, 10.f, FColor::Red, TEXT("No Weapon Manager was given to the player !"));
		return;
	}
	
	WeaponManager = Manager;
	WeaponManager->Initialize(ArmsMesh);
}

void ALMPlayer::PlayAnimation(UAnimMontage* Animation)
{
}

void ALMPlayer::FireBullets(const FLMBulletInfo Settings)
{
	FVector EndLocation;
	FVector ShotVector;
	FHitResult OutHit = FHitResult();

	DrawDebugLine(GetWorld(), Settings.Origin, Settings.Origin + (Settings.Direction * Settings.MaxDistance), FColor::Blue, false, 2.f);
	
	for (int Shots = 0; Shots < Settings.BulletCount; Shots++)
	{
		ShotVector = UKismetMathLibrary::RandomUnitVectorInConeInDegreesFromStream(SpreadStream, Settings.Direction, Settings.Spread);
		EndLocation = Settings.Origin + (ShotVector * Settings.MaxDistance);

		const bool HasHit = GetWorld()->LineTraceSingleByChannel(OutHit, Settings.Origin, EndLocation, TRACE_BULLET);
		DrawDebugLineTraceSingle(GetWorld(), Settings.Origin, EndLocation, EDrawDebugTrace::ForDuration, HasHit, OutHit, FColor::Green, FColor::Red, 2.f);
		
		if (!HasHit)
			continue;
		
		FString Hit = FString::Printf(TEXT("Hit %s"), *OutHit.Component->GetName());
		// GEngine->AddOnScreenDebugMessage(INDEX_NONE, 2.f, FColor::Red, Hit);
		UE_LOG(LogTemp, Display, TEXT("%s"), *Hit);
		
		if (ULMHitBox* HitBox = Cast<ULMHitBox>(OutHit.Component))
		{
			UE_LOG(LogTemp, Warning, TEXT("Hit Hitbox"));
			HitBox->OnHit(Settings);
		}
	}
}

FVector ALMPlayer::GetWeaponFiringOrigin() const
{
	if (!Camera)
	{
		GEngine->AddOnScreenDebugMessage(INDEX_NONE, 10.f, FColor::Silver, TEXT("No camera ???"));
		return FVector::ZeroVector;
	}
	return Camera->GetComponentTransform().GetLocation();
}
FVector ALMPlayer::GetAimVector() const { return Camera->GetForwardVector(); }


void ALMPlayer::Tick(float DeltaTime)
{
	Super::Tick(DeltaTime);
}

void ALMPlayer::SetupPlayerInputComponent(UInputComponent* PlayerInputComponent)
{
	Super::SetupPlayerInputComponent(PlayerInputComponent);

}

